#!/usr/bin/perl
########################################################################
#
# $Id: click2call.pl,v 1.6 2012-12-21 15:17:30 gosha Exp $
#
#		(c) Okunev Igor <igor[at]prv.mts-nn.ru>
#					2012
#
########################################################################
package VirtualPBX;

open STDERR, '>>/var/log/VirtualPBX/XVB.stderr';

use strict;

use lib qw( /opt/VirtualPBX/lib );

use XVB::Core;
use XVB::Database;
use XVB::Routes;
use XVB::CIDFilters;
use XVB::MC;
use XVB::UI::AGI::Calls;
use XVB::Users;
use XVB::Mailboxes;
use XVB::UI::CGI;
use XVB::UI::CGI::Click2Call;
use XVB::DIDs;
use XVB::CDR;

use CGI;
use Digest::MD5 qw(md5_hex);
use MIME::Base64 qw(encode_base64);
use Fcntl qw(:flock);
use POSIX qw(SIGALRM);

#
# Asterisk Spool dir
#
$ENV{'AST_SPOOL_DIR'} = '/var/spool/asterisk/';

use vars qw( $CoreObj );

########################################################################
							main();
########################################################################


########################################################################
#
sub new {
	my ( $class, $config_file ) = @_;

	unless ( $CoreObj ) {
		$CoreObj = bless {
							_CONF			=> {},
						}, $class;

		$CoreObj->core_cfg_init( $config_file );

		$CoreObj->core_log_init();

		#
		# tune DB connection type
		#
		$CoreObj->{'_DB'}->{'PERSISTENT_CONNECTION'} = 1;

		#
		# Disabled Radius acct
		#
		$CoreObj->{'_CONF'}->{'radius_acct_host'} = 0;

		#
		# CGI POST LIMIT
		#
		$CGI::POST_MAX = $CoreObj->core_cfg_get( 'CGI_POST_MAX' ) || 100_000;

		#
		# work dir
		#
		chdir($CoreObj->core_cfg_get( 'PATH_WORK_DIR' ));
	
		if ( $CoreObj->core_cfg_get('GEARMAN_HOST') ) {
			eval 'use XVB::Gearman';
			$CoreObj->gearman_init();
			foreach my $method ( split(/\s*,\s*/, $CoreObj->core_cfg_get('GEARMAN_FUNC') ) ) {
				import XVB::Gearman lc($method);
				$CoreObj->{'_GEARMAN'}->{'FUNC'}->{lc($method)} = 1;
			}
		} else {
			eval 'use XVB::Notify::HTTP; XVB::Notify::GA; use XVB::Radius;';
			if ( $CoreObj->core_cfg_get('RADIUS_ACCT_HOST') ) {
				$CoreObj->radius_init();
			}
		}
	}
	
	$CoreObj->{'_CDR'}			= {};
	$CoreObj->{'_USER_CREDS'}	= {};
	$CoreObj->{'_USER_VBOXES'}	= {};
	$CoreObj->{'_SESSION'}		= {};
	$CoreObj->{'_CGI'}			= new CGI;

	return $CoreObj;
}

########################################################################
#
sub main {
	my $obj = new VirtualPBX( '/opt/VirtualPBX/etc/xvb.cfg' );

	my $key = $obj->cgi_param('key');
	my $ac = $obj->cgi_param('ac');
	my $phone = $obj->cgi_param('ph');
	
	if ( ! length($key) or ! length($ac) or ! length($phone) ) {
		return $obj->cgi_response( 200, 'Ok', undef, 'ERR: Missing param' );
	}

	my $tmp_file = 'click2call-'. ( $ENV{'REMOTE_HOST'} || '0.0.0.0' );

	if ( -e  $ENV{'AST_SPOOL_DIR'} . $tmp_file ) {
		return $obj->cgi_response( 200, 'Ok', undef, 'ERR: Call exists' );
	}

	my $rc = $obj->cache( "sh-short-user-$ac" );
	unless ( ref($rc) ) {
		return $obj->cgi_response( 200, 'Ok', undef, 'ERR: Unknown USER' );
	}

	my $user_ref = $obj->user_get_info( $rc->{'VOICENUMBER'}, $ac );
	unless ( $user_ref ) {
		return $obj->cgi_response( 200, 'Ok', undef, 'ERR: Unknown USER' );
	}

	my $c2c = $obj->click2call_get();

	my $ext = $c2c->{$key}->{'EXTENSION'};
	unless ( length($ext) ) {
		return $obj->cgi_response( 200, 'Ok', undef, 'ERR: Auth error' );
	}

	my ( $npn, $cid, $gw, $tlimit, $rn ) = $obj->call_build_number( $phone, $ext );

	$obj->cdr_set();
	$obj->cdr_set( {
			CALLED_ID	=> $phone,
			CALLER_ID	=> $ENV{'REMOTE_ADDR'},
			CALL_TYPE	=> 'web',
			EVENT_TYPE	=> 'Click2Call',
			EVENT_EXT	=> $ext,
			ENH_CALLID	=> $obj->{'_CDR'}->{'CALL_ID'},
			ENH_STAGE	=> 'init'
		} );

	if ( $obj->create_call_file( $npn, $ext, $cid, $rn, $gw, $tlimit, $tmp_file ) ) {
		$obj->cdr_write();
		return $obj->cgi_response( 200, 'Ok', undef, 'OK' );
	} else {
		return $obj->cgi_response( 200, 'Ok', undef, 'ERR: Internal error' );
	}
}

#
# create call file
#
sub create_call_file {
	my ( $obj, $dial_str, $exten, $caller_id, $rn, $gw, $tlimit, $tmp_file ) = @_;

	my $call_str = join( 	'*',
							$obj->core_creds()->{'VOICENUMBER'},
							$obj->core_creds()->{'ACCESS_CODE'},
							$exten || 0 );

	#
	# ring timeout 10-120 sec
	#
	my $ring_timeout = 30;

	my $cf_data =	"Channel: $dial_str\n".
					"Context: xvb-click2call\n".
					"Extension: $call_str\n".
					"CallerID: $call_str\n".
					'WaitTime: '. $ring_timeout ."\n".
					'Set: CID_'. md5_hex($caller_id) .'='. encode_base64( $gw, '' ). "\n".
					'Set: TLIMIT_'. md5_hex($caller_id) .'='. int($tlimit). "\n".
					'Set: __CB_CALL_ID='.  $obj->{'_CDR'}->{'CALL_ID'} ."\n".
					'Set: __XVB_IP='. $ENV{'REMOTE_HOST'} . "\n".
					'Set: __XVB_AC='. $obj->core_creds()->{'ACCESS_CODE'} ."\n".
					'Set: __XVB_SUBSCR_ID='. $obj->core_creds()->{'ID'} ."\n".
					'Set: __XVB_EXT_ID='. $exten ."\n".
					'Set: __XVB_PN='. $rn ."\n";

	return $obj->call_callfile( $cf_data, $tmp_file, 0, 1 );
}

